package com.logicbig.example;

import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import javax.ws.rs.ext.Providers;
import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.List;

@Provider
@Produces("text/csv")
public class CsvWriter<T> implements MessageBodyWriter<List<T>> {
    @Context
    Providers providers;

    @Override
    public void writeTo(List<T> ts, Class<?> type, Type genericType, Annotation[] annotations,
                        MediaType mediaType, MultivaluedMap<String, Object> httpHeaders,
                        OutputStream entityStream) throws IOException, WebApplicationException {
        String response = "";
        for (T t : ts) {
            try {
                Class<?> c = t.getClass();
                //using Java Bean API to access property values
                BeanInfo beanInfo = Introspector.getBeanInfo(c);
                String temp = "";
                for (PropertyDescriptor pd : beanInfo.getPropertyDescriptors()) {
                    if (pd.getName().equals("class")) {//skip getClass()
                        continue;
                    }
                    Object o = pd.getReadMethod().invoke(t);
                    temp += "," + o;
                }
                if (temp.length() > 0) {
                    temp = temp.substring(1);
                    response += temp + "\n";
                }
            } catch (Exception e) {
                throw new WebApplicationException(e);
            }
        }
        //using Providers instance
        MessageBodyWriter<String> plainTextWriter = providers.getMessageBodyWriter(String.class,
                genericType, annotations, MediaType.TEXT_PLAIN_TYPE);
        plainTextWriter.writeTo(response, String.class, genericType, annotations, mediaType,
                httpHeaders, entityStream);
    }

    @Override
    public boolean isWriteable(Class<?> type, Type genericType,
                               Annotation[] annotations, MediaType mediaType) {
        return true;
    }

    @Override
    public long getSize(List<T> ts, Class<?> type, Type genericType,
                        Annotation[] annotations, MediaType mediaType) {
        return -1;
    }
}